@@ -55,6 +55,19 @@ class MechanismDriver(object):
     methods that are part of the database transaction.
     """
 
+    # Used in generating resource provider UUIDs for physical network
+    # interfaces. Each mechanism driver supporting Placement should have its
+    # own UUID v5 namespace (which is a UUID v1 in turn). When set to a
+    # concrete value use a uuid.UUID() object, not the string format.
+    # It will be used when hashing RP UUIDs from:
+    #     (mech driver namespace, hostname, physical bridge/interface name)
+    # When needed generate new namespace UUIDs by:
+    #     python -c 'import uuid ; print uuid.uuid1()'
+    # See also:
+    #     https://tools.ietf.org/html/rfc4122
+    #     https://stackoverflow.com/a/7816117
+    resource_provider_uuid5_namespace = None
+
     @abc.abstractmethod
     def initialize(self):
         """Perform driver initialization.
@@ -69,7 +82,7 @@ class MechanismDriver(object):
         """Allocate resources for a new network.
 
         :param context: NetworkContext instance describing the new
-        network.
+            network.
 
         Create a new network, allocating resources as necessary in the
         database. Called inside transaction context on session. Call
@@ -82,7 +95,7 @@ class MechanismDriver(object):
         """Create a network.
 
         :param context: NetworkContext instance describing the new
-        network.
+            network.
 
         Called after the transaction commits. Call can block, though
         will block the entire process so care should be taken to not
@@ -95,8 +108,8 @@ class MechanismDriver(object):
         """Update resources of a network.
 
         :param context: NetworkContext instance describing the new
-        state of the network, as well as the original state prior
-        to the update_network call.
+            state of the network, as well as the original state prior
+            to the update_network call.
 
         Update values of a network, updating the associated resources
         in the database. Called inside transaction context on session.
@@ -113,8 +126,8 @@ class MechanismDriver(object):
         """Update a network.
 
         :param context: NetworkContext instance describing the new
-        state of the network, as well as the original state prior
-        to the update_network call.
+            state of the network, as well as the original state prior
+            to the update_network call.
 
         Called after the transaction commits. Call can block, though
         will block the entire process so care should be taken to not
@@ -131,7 +144,7 @@ class MechanismDriver(object):
         """Delete resources for a network.
 
         :param context: NetworkContext instance describing the current
-        state of the network, prior to the call to delete it.
+            state of the network, prior to the call to delete it.
 
         Delete network resources previously allocated by this
         mechanism driver for a network. Called inside transaction
@@ -145,7 +158,7 @@ class MechanismDriver(object):
         """Delete a network.
 
         :param context: NetworkContext instance describing the current
-        state of the network, prior to the call to delete it.
+            state of the network, prior to the call to delete it.
 
         Called after the transaction commits. Call can block, though
         will block the entire process so care should be taken to not
@@ -159,7 +172,7 @@ class MechanismDriver(object):
         """Allocate resources for a new subnet.
 
         :param context: SubnetContext instance describing the new
-        subnet.
+            subnet.
 
         Create a new subnet, allocating resources as necessary in the
         database. Called inside transaction context on session. Call
@@ -172,7 +185,7 @@ class MechanismDriver(object):
         """Create a subnet.
 
         :param context: SubnetContext instance describing the new
-        subnet.
+            subnet.
 
         Called after the transaction commits. Call can block, though
         will block the entire process so care should be taken to not
@@ -185,8 +198,8 @@ class MechanismDriver(object):
         """Update resources of a subnet.
 
         :param context: SubnetContext instance describing the new
-        state of the subnet, as well as the original state prior
-        to the update_subnet call.
+            state of the subnet, as well as the original state prior
+            to the update_subnet call.
 
         Update values of a subnet, updating the associated resources
         in the database. Called inside transaction context on session.
@@ -203,8 +216,8 @@ class MechanismDriver(object):
         """Update a subnet.
 
         :param context: SubnetContext instance describing the new
-        state of the subnet, as well as the original state prior
-        to the update_subnet call.
+            state of the subnet, as well as the original state prior
+            to the update_subnet call.
 
         Called after the transaction commits. Call can block, though
         will block the entire process so care should be taken to not
@@ -221,7 +234,7 @@ class MechanismDriver(object):
         """Delete resources for a subnet.
 
         :param context: SubnetContext instance describing the current
-        state of the subnet, prior to the call to delete it.
+            state of the subnet, prior to the call to delete it.
 
         Delete subnet resources previously allocated by this
         mechanism driver for a subnet. Called inside transaction
@@ -235,7 +248,7 @@ class MechanismDriver(object):
         """Delete a subnet.
 
         :param context: SubnetContext instance describing the current
-        state of the subnet, prior to the call to delete it.
+            state of the subnet, prior to the call to delete it.
 
         Called after the transaction commits. Call can block, though
         will block the entire process so care should be taken to not
@@ -273,8 +286,8 @@ class MechanismDriver(object):
         """Update resources of a port.
 
         :param context: PortContext instance describing the new
-        state of the port, as well as the original state prior
-        to the update_port call.
+            state of the port, as well as the original state prior
+            to the update_port call.
 
         Called inside transaction context on session to complete a
         port update as defined by this mechanism driver. Raising an
@@ -290,8 +303,8 @@ class MechanismDriver(object):
         """Update a port.
 
         :param context: PortContext instance describing the new
-        state of the port, as well as the original state prior
-        to the update_port call.
+            state of the port, as well as the original state prior
+            to the update_port call.
 
         Called after the transaction completes. Call can block, though
         will block the entire process so care should be taken to not
@@ -308,7 +321,7 @@ class MechanismDriver(object):
         """Delete resources of a port.
 
         :param context: PortContext instance describing the current
-        state of the port, prior to the call to delete it.
+            state of the port, prior to the call to delete it.
 
         Called inside transaction context on session. Runtime errors
         are not expected, but raising an exception will result in
@@ -320,7 +333,7 @@ class MechanismDriver(object):
         """Delete a port.
 
         :param context: PortContext instance describing the current
-        state of the port, prior to the call to delete it.
+            state of the port, prior to the call to delete it.
 
         Called after the transaction completes. Call can block, though
         will block the entire process so care should be taken to not
@@ -413,6 +426,27 @@ class MechanismDriver(object):
         """
         return candidate_hosts
 
+    def responsible_for_ports_allocation(self, context):
+        """Is responsible for a port's resource provider?
+
+        :param context: PortContext instance describing the port
+        :returns: True for responsible, False for not responsible
+
+        For ports having an allocation in Placement (as expressed
+        in the port's binding:profile.allocation) decide while
+        binding if this mechanism driver is responsible for the
+        physical network interface represented by the resource
+        provider in Placement. Find the resource provider UUID in
+        context.current['binding:profile']['allocation'].
+
+        Drivers wanting to support resource allocations for ports in
+        Placement (eg. wanting to guarantee some minimum bandwidth)
+        must implement this method.
+
+        Default implementation returns False (backward compatibility).
+        """
+        return False
+
 
 @six.add_metaclass(abc.ABCMeta)
 class _TypeDriverBase(object):
@@ -1037,6 +1071,16 @@ class ExtensionDriver(object):
         """
         pass
 
+    @property
+    def extension_aliases(self):
+        """List of extension aliases supported by the driver.
+
+        Return a list of aliases identifying the core API extensions
+        supported by the driver. By default this just returns the
+        extension_alias property for backwards compatibility.
+        """
+        return [self.extension_alias]
+
     def process_create_network(self, plugin_context, data, result):
         """Process extended attributes for create network.
 
