@@ -723,6 +723,31 @@ class TestAttributeValidation(base.BaseTestCase):
     def test_validate_subnet(self):
         self._test_validate_subnet(validators.validate_subnet)
 
+    def test_validate_route_cidr(self):
+        # Valid - CIDR
+        cidr = "10.0.0.0/8"
+        msg = validators.validate_route_cidr(cidr, None)
+        self.assertIsNone(msg)
+
+        # Valid - CIDR
+        cidr = "192.168.1.1/32"
+        msg = validators.validate_route_cidr(cidr, None)
+        self.assertIsNone(msg)
+
+        # Invalid - CIDR
+        cidr = "192.168.1.1/8"
+        msg = validators.validate_route_cidr(cidr, None)
+        error = _("'%(data)s' is not a recognized CIDR,"
+                  " '%(cidr)s' is recommended") % {"data": cidr,
+                                                   "cidr": "192.0.0.0/8"}
+        self.assertEqual(error, msg)
+
+        # Invalid - CIDR format error
+        cidr = 'invalid'
+        msg = validators.validate_route_cidr(cidr, None)
+        error = "'%s' is not a valid CIDR" % cidr
+        self.assertEqual(error, msg)
+
     def test_validate_subnet_or_none(self):
         self._test_validate_subnet(validators.validate_subnet_or_none,
                                    allow_none=True)
@@ -856,6 +881,16 @@ class TestAttributeValidation(base.BaseTestCase):
                 mock.Mock(return_value=None), list_obj)
             self.assertIsNone(msg)
 
+    def test__test__validate_list_of_items_non_empty(self):
+        items = None
+        msg = validators._validate_list_of_items_non_empty(mock.Mock(), items)
+        error = "'%s' is not a list" % items
+        self.assertEqual(error, msg)
+
+        items = []
+        msg = validators._validate_list_of_items_non_empty(mock.Mock(), items)
+        self.assertEqual("List should not be empty", msg)
+
     def test_validate_dict_type(self):
         for value in (None, True, '1', []):
             self.assertEqual("'%s' is not a dictionary" % value,
@@ -880,7 +915,7 @@ class TestAttributeValidation(base.BaseTestCase):
 
         constraints['key1'] = {'type:unsupported': None, 'required': True}
         msg = validators.validate_dict(dictionary, constraints)
-        self.assertEqual("Validator 'type:unsupported' does not exist.", msg)
+        self.assertEqual("Validator 'type:unsupported' does not exist", msg)
 
     def test_validate_dict_not_required_keys(self):
         dictionary, constraints = self._construct_dict_and_constraints()
@@ -1018,12 +1053,19 @@ class TestAttributeValidation(base.BaseTestCase):
     def test_validate_subports_invalid_duplicate_segmentation_id(self):
         body = [
             {'port_id': '00000000-ffff-ffff-ffff-000000000000',
-             'segmentation_id': '3', 'segmentation_type': 'vlan'},
+             'segmentation_id': 0, 'segmentation_type': 'vlan'},
             {'port_id': '11111111-ffff-ffff-ffff-000000000000',
-             'segmentation_id': '3', 'segmentation_type': 'vlan'}
+             'segmentation_id': 0, 'segmentation_type': 'vlan'}
         ]
         self.assertIsNotNone(validators.validate_subports(body))
 
+    def test_validate_subports_with_segmentation_id_0(self):
+        body = [
+            {'port_id': '00000000-ffff-ffff-ffff-000000000000',
+             'segmentation_id': '0', 'segmentation_type': 'vlan'}
+        ]
+        self.assertIsNone(validators.validate_subports(body))
+
     def test_validate_subports_inherit_segmentation_details(self):
         body = [
             {'port_id': '00000000-ffff-ffff-ffff-000000000000',
@@ -1118,47 +1160,51 @@ class TestPortRangeValidation(base.BaseTestCase):
         self.assertIsNone(result)
 
     def test_valid_range(self):
-        result = validators.validate_port_range_or_none("80:8888")
+        # NOTE(huntxu): This case would fail when ports are compared as
+        # strings, since '9' > '1111'.
+        result = validators.validate_port_range_or_none("9:1111")
         self.assertIsNone(result)
 
     def test_port_too_high(self):
         result = validators.validate_port_range_or_none("99999")
-        self.assertEqual(u"Invalid port: 99999.", result)
+        self.assertEqual(u"Invalid port: 99999", result)
 
     def test_port_too_low(self):
         result = validators.validate_port_range_or_none("-1")
-        self.assertEqual(u"Invalid port: -1.", result)
+        self.assertEqual(u"Invalid port: -1", result)
 
     def test_range_too_high(self):
         result = validators.validate_port_range_or_none("80:99999")
-        self.assertEqual(u"Invalid port: 99999.", result)
+        self.assertEqual(u"Invalid port: 99999", result)
 
     def test_range_too_low(self):
         result = validators.validate_port_range_or_none("-1:8888")
-        self.assertEqual(u"Invalid port: -1.", result)
+        self.assertEqual(u"Invalid port: -1", result)
 
     def test_range_wrong_way(self):
-        result = validators.validate_port_range_or_none("8888:80")
+        # NOTE(huntxu): This case would fail when ports are compared as
+        # strings, since '1111' < '9'.
+        result = validators.validate_port_range_or_none("1111:9")
         self.assertEqual(u"First port in a port range must be lower than the "
-                         "second port.", result)
+                         "second port", result)
 
     def test_range_invalid(self):
         result = validators.validate_port_range_or_none("DEAD:BEEF")
-        self.assertEqual(u"Invalid port: DEAD.", result)
+        self.assertEqual(u"Invalid port: DEAD", result)
 
     def test_range_bad_input(self):
         result = validators.validate_port_range_or_none(['a', 'b', 'c'])
-        self.assertEqual(u"Invalid port: ['a', 'b', 'c'].", result)
+        self.assertEqual(u"Invalid port: ['a', 'b', 'c']", result)
 
     def test_range_colon(self):
         result = validators.validate_port_range_or_none(":")
         self.assertEqual(u"Port range must be two integers separated by a "
-                         "colon.", result)
+                         "colon", result)
 
     def test_too_many_colons(self):
         result = validators.validate_port_range_or_none("80:888:8888")
         self.assertEqual(u"Port range must be two integers separated by a "
-                         "colon.", result)
+                         "colon", result)
 
 
 class TestAnyKeySpecs(base.BaseTestCase):
